
/* 
 * This is a texture plugin for blender 1.60 or above.
 *
 * Dot-drawing plugin, based on a sample plugin from NaN
 * Copyright (c) 1999, Bernd Breitenbach
 *
 * $Id: dots.c,v 1.4 1999/05/06 19:26:33 bernd Exp bernd $
 * 
 */

/* ************************************************************
	NOTE: This software comes with absolute no warranty.
	You may distribute, modify and use it free of charge.
	
	If you have any enhancements or bug fixes, please let me now:

	              breitenb@iml.fhg.de
   ************************************************************ */

 
#include "math.h"
#include "plugin.h"

/* ******************** GLOBAL VARIABLES ***************** */

char name[]= "Dots";

/* Subtype names must be less than 15 characters */

#define NR_TYPES	1
char stnames[NR_TYPES][16]= {"Dots"};

VarStruct varstr[]= {
	 NUM|FLO,	"Size",			0.25,	 0.0, 1.0,  "The size of each dot", 
	 NUM|FLO,	"Noise",		0.0,	 0.0, 10.0, "Noise parameter: 0. means no noise",
	 TOG|INT,	"Smooth",	        0.0,	 0.0, 1.0,  "Smooth dots",
	 TOG|INT,	"No AAlias",	        0.0,	 0.0, 1.0,  "Turn antialias off (independent from render settings)",
};

/* The cast struct is for input in the main doit function
   Varstr and Cast must have the same variables in the same order */ 

typedef struct Cast {
	float size;
	float noise;
        int  smooth;
        int  noaalias;
} Cast;

/* result: 
   Intensity, R, G, B, Alpha, nor.x, nor.y, nor.z
 */

float result[8];

/* cfra: the current frame */

float cfra;

int plugin_tex_doit(int, Cast *, float *, float *, float *);


/* ******************** Fixed functions ***************** */

int plugin_tex_getversion(void) 
{	
	return B_PLUGIN_VERSION;
}

void plugin_but_changed(int but) 
{
}

void plugin_init(void)
{
}

/* this function should not be changed: */

void plugin_getinfo(PluginInfo *info)
{
	info->name= name;
	info->stypes= NR_TYPES;
	info->nvars= sizeof(varstr)/sizeof(VarStruct);
	
	info->snames= stnames[0];
	info->result= result;
	info->cfra= &cfra;
	info->varstr= varstr;

	info->init= plugin_init;
	info->tex_doit=  (TexDoit) plugin_tex_doit;
	info->callback= plugin_but_changed;
}

/* ************************************************************
	Dots
	
	This function placing  dots of size 'size' on the vertices 
	of a regular triangle lattice covering the x-y-plane. 
	With 'smooth' you can choose between dots with a sharp border, 
	or smooth dots with a texture value that is raising
	from 0. at the border to 1. at the center.

	Apart from smoothing dots, anti-aliasing is performed by
	calculating the mean value from four texture values at
	the vertices of an pixel.
   ************************************************************ */

#define SQRT3 (1.73205080756888)
#define SQRT3_2 (SQRT3/2.)

float calc_dot_value(float x,float y,float size,int smooth)
{ 
  float size2=size*size,dist2;
  float retval=0.;

  /* First we do a little origami: */

  x=fabs(x);
  y=fabs(y);

  x=x-ffloor(x);
  y=y-ffloor(y/SQRT3)*SQRT3;

  if (x>0.5)     x=1.-x;
  if (y>SQRT3_2) y=SQRT3-y;

  /* No we have to check only two circles at (0,0) and (1/2,sqrt(3)/2) */

  dist2=x*x+y*y;
  if (dist2<=size2) {
    if (smooth) {
      retval=sqrt(size2-dist2)/size;
    } else {
      retval=1.;
    }
  } else {
    x=0.5-x;
    y=SQRT3_2-y;
    dist2=x*x+y*y;
    if (dist2<=size2) {
      if (smooth) 
	retval=sqrt(size2-dist2)/size;
      else
	retval=1.;
    }
  }
  return retval;
}



int plugin_tex_doit(int stype, Cast *cast, float *texvec, float *dxt, float *dyt)
{
  float x,y;

  x=texvec[0];
  y=texvec[1];
  if(cast->noise>0.001) {
    x+= hnoise(cast->noise, texvec[0], texvec[1], texvec[2]);
    y+= hnoise(cast->noise, texvec[1], texvec[2], texvec[0]);
  }

  result[0]=calc_dot_value(x,y,cast->size,cast->smooth);
  if (!cast->noaalias && dxt && dyt) {
    float offx=fabs(dxt[0])+fabs(dxt[1]);
    float offy=fabs(dyt[0])+fabs(dyt[1]);
    result[0]+=calc_dot_value(x+offx,y,cast->size,cast->smooth);
    result[0]+=calc_dot_value(x,y+offy,cast->size,cast->smooth);
    result[0]+=calc_dot_value(x+offx,y+offy,cast->size,cast->smooth);
    result[0]/=4.0;
  }

  return 0;
}





